@@ -14,10 +14,10 @@ start_link(Server, Listen, Loop) ->
     proc_lib:spawn_link(?MODULE, init, [Server, Listen, Loop]).
 
 init(Server, Listen, Loop) ->
-    T1 = now(),
+    T1 = os:timestamp(),
     case catch mochiweb_socket:accept(Listen) of
         {ok, Socket} ->
-            gen_server:cast(Server, {accepted, self(), timer:now_diff(now(), T1)}),
+            gen_server:cast(Server, {accepted, self(), timer:now_diff(os:timestamp(), T1)}),
             call_loop(Loop, Socket);
         {error, closed} ->
             exit(normal);
