@@ -17,6 +17,7 @@
 -module(rabbit_exchange_type_consistent_hash_test).
 -export([test/0]).
 -include_lib("amqp_client/include/amqp_client.hrl").
+-include_lib("eunit/include/eunit.hrl").
 
 %% Because the routing is probabilistic, we can't really test a great
 %% deal here.
@@ -29,6 +30,8 @@ test() ->
 t(Qs) ->
     ok = test_with_rk(Qs),
     ok = test_with_header(Qs),
+    ok = test_binding_with_negative_routing_key(),
+    ok = test_binding_with_non_numeric_routing_key(),
     ok.
 
 test_with_rk(Qs) ->
@@ -63,19 +66,19 @@ test0(MakeMethod, MakeMsg, DeclareArgs, [Q1, Q2, Q3, Q4] = Queues) ->
                             type = <<"x-consistent-hash">>,
                             auto_delete = true,
                             arguments = DeclareArgs
-                           }),
+                          }),
     [#'queue.declare_ok'{} =
          amqp_channel:call(Chan, #'queue.declare' {
-                             queue = Q, exclusive = true }) || Q <- Queues],
+                             queue = Q, exclusive = true}) || Q <- Queues],
     [#'queue.bind_ok'{} =
-         amqp_channel:call(Chan, #'queue.bind' { queue = Q,
+         amqp_channel:call(Chan, #'queue.bind' {queue = Q,
                                                  exchange = <<"e">>,
-                                                 routing_key = <<"10">> })
+                                                 routing_key = <<"10">>})
      || Q <- [Q1, Q2]],
     [#'queue.bind_ok'{} =
-         amqp_channel:call(Chan, #'queue.bind' { queue = Q,
+         amqp_channel:call(Chan, #'queue.bind' {queue = Q,
                                                  exchange = <<"e">>,
-                                                 routing_key = <<"20">> })
+                                                 routing_key = <<"20">>})
      || Q <- [Q3, Q4]],
     #'tx.select_ok'{} = amqp_channel:call(Chan, #'tx.select'{}),
     [amqp_channel:call(Chan,
@@ -86,13 +89,49 @@ test0(MakeMethod, MakeMsg, DeclareArgs, [Q1, Q2, Q3, Q4] = Queues) ->
         [begin
             #'queue.declare_ok'{message_count = M} =
                  amqp_channel:call(Chan, #'queue.declare' {queue     = Q,
-                                                           exclusive = true }),
+                                                           exclusive = true}),
              M
          end || Q <- Queues],
     Count = lists:sum(Counts), %% All messages got routed
     [true = C > 0.01 * Count || C <- Counts], %% We are not *grossly* unfair
-    amqp_channel:call(Chan, #'exchange.delete' { exchange = <<"e">> }),
-    [amqp_channel:call(Chan, #'queue.delete' { queue = Q }) || Q <- Queues],
+    amqp_channel:call(Chan, #'exchange.delete' {exchange = <<"e">>}),
+    [amqp_channel:call(Chan, #'queue.delete' {queue = Q}) || Q <- Queues],
     amqp_channel:close(Chan),
     amqp_connection:close(Conn),
     ok.
+
+test_binding_with_negative_routing_key() ->
+    {ok, Conn} = amqp_connection:start(#amqp_params_network{}),
+    {ok, Chan} = amqp_connection:open_channel(Conn),
+    Declare1 = #'exchange.declare'{exchange = <<"bind-fail">>,
+                                    type = <<"x-consistent-hash">>},
+    #'exchange.declare_ok'{} = amqp_channel:call(Chan, Declare1),
+    Q = <<"test-queue">>,
+    Declare2 = #'queue.declare'{queue = Q},
+    #'queue.declare_ok'{} = amqp_channel:call(Chan, Declare2),
+    process_flag(trap_exit, true),
+    Cmd = #'queue.bind'{exchange = <<"bind-fail">>,
+                         routing_key = <<"-1">>},
+    ?assertExit(_, amqp_channel:call(Chan, Cmd)),
+    {ok, Ch2} = amqp_connection:open_channel(Conn),
+    amqp_channel:call(Ch2, #'queue.delete'{queue = Q}),
+    amqp_connection:close(Conn),
+    ok.
+
+test_binding_with_non_numeric_routing_key() ->
+    {ok, Conn} = amqp_connection:start(#amqp_params_network{}),
+    {ok, Chan} = amqp_connection:open_channel(Conn),
+    Declare1 = #'exchange.declare'{exchange = <<"bind-fail">>,
+                                    type = <<"x-consistent-hash">>},
+    #'exchange.declare_ok'{} = amqp_channel:call(Chan, Declare1),
+    Q = <<"test-queue">>,
+    Declare2 = #'queue.declare'{queue = Q},
+    #'queue.declare_ok'{} = amqp_channel:call(Chan, Declare2),
+    process_flag(trap_exit, true),
+    Cmd = #'queue.bind'{exchange = <<"bind-fail">>,
+                         routing_key = <<"not-a-number">>},
+    ?assertExit(_, amqp_channel:call(Chan, Cmd)),
+    {ok, Ch2} = amqp_connection:open_channel(Conn),
+    amqp_channel:call(Ch2, #'queue.delete'{queue = Q}),
+    amqp_connection:close(Conn),
+    ok.
