@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Console.
 %%
 %%   The Initial Developer of the Original Code is GoPivotal, Inc.
-%%   Copyright (c) 2010-2014 GoPivotal, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2015 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_external_stats).
@@ -24,6 +24,8 @@
 
 -export([list_registry_plugins/1]).
 
+-import(rabbit_misc, [pget/2]).
+
 -include_lib("rabbit_common/include/rabbit.hrl").
 
 -define(REFRESH_RATIO, 5000).
@@ -34,11 +36,11 @@
                uptime, run_queue, processors, exchange_types,
                auth_mechanisms, applications, contexts,
                log_file, sasl_log_file, db_dir, config_files, net_ticktime,
-               enabled_plugins]).
+               enabled_plugins, persister_stats]).
 
 %%--------------------------------------------------------------------
 
--record(state, {fd_total}).
+-record(state, {fd_total, fhc_stats, fhc_stats_derived, node_owners}).
 
 %%--------------------------------------------------------------------
 
@@ -183,6 +185,7 @@ i(sasl_log_file,   _State) -> log_location(sasl);
 i(db_dir,          _State) -> list_to_binary(rabbit_mnesia:dir());
 i(config_files,    _State) -> [list_to_binary(F) || F <- rabbit:config_files()];
 i(net_ticktime,    _State) -> net_kernel:get_net_ticktime();
+i(persister_stats,  State) -> persister_stats(State);
 i(enabled_plugins, _State) -> {ok, Dir} = application:get_env(
                                            rabbit, enabled_plugins_file),
                               rabbit_plugins:read_enabled(Dir);
@@ -224,6 +227,51 @@ set_plugin_name(Name, Module) ->
     [{name, list_to_binary(atom_to_list(Name))} |
      proplists:delete(name, Module:description())].
 
+persister_stats(#state{fhc_stats         = FHC,
+                       fhc_stats_derived = FHCD}) ->
+    [{flatten_key(K), V} || {{_Op, Type} = K, V} <- FHC,
+                            Type =/= time] ++
+        [{flatten_key(K), V} || {K, V} <- FHCD].
+
+flatten_key({A, B}) ->
+    list_to_atom(atom_to_list(A) ++ "_" ++ atom_to_list(B)).
+
+cluster_links() ->
+    {ok, Items} = net_kernel:nodes_info(),
+    [Link || Item <- Items,
+             Link <- [format_nodes_info(Item)], Link =/= undefined].
+
+format_nodes_info({Node, Info}) ->
+    Owner = proplists:get_value(owner, Info),
+    case catch process_info(Owner, links) of
+        {links, Links} ->
+            case [Link || Link <- Links, is_port(Link)] of
+                [Port] ->
+                    {Node, Owner, format_nodes_info1(Port)};
+                _ ->
+                    undefined
+            end;
+        _ ->
+            undefined
+    end.
+
+format_nodes_info1(Port) ->
+    case {rabbit_net:socket_ends(Port, inbound),
+          rabbit_net:getstat(Port, [recv_oct, send_oct])} of
+        {{ok, {PeerAddr, PeerPort, SockAddr, SockPort}}, {ok, Stats}} ->
+            [{peer_addr, maybe_ntoab(PeerAddr)},
+             {peer_port, PeerPort},
+             {sock_addr, maybe_ntoab(SockAddr)},
+             {sock_port, SockPort},
+             {recv_bytes, pget(recv_oct, Stats)},
+             {send_bytes, pget(send_oct, Stats)}];
+        _ ->
+            []
+    end.
+
+maybe_ntoab(A) when is_tuple(A) -> list_to_binary(rabbit_misc:ntoab(A));
+maybe_ntoab(H)                  -> H.
+
 %%--------------------------------------------------------------------
 
 %% This is slightly icky in that we introduce knowledge of
@@ -256,17 +304,18 @@ format_mochiweb_option_list(C) ->
 
 format_mochiweb_option(ssl_opts, V) ->
     format_mochiweb_option_list(V);
-format_mochiweb_option(ciphers, V) ->
-    list_to_binary(rabbit_misc:format("~w", [V]));
-format_mochiweb_option(_K, V) when is_list(V) ->
-    list_to_binary(V);
 format_mochiweb_option(_K, V) ->
-    V.
+    case io_lib:printable_list(V) of
+        true  -> list_to_binary(V);
+        false -> list_to_binary(rabbit_misc:format("~w", [V]))
+    end.
 
 %%--------------------------------------------------------------------
 
 init([]) ->
-    State = #state{fd_total = file_handle_cache:ulimit()},
+    State = #state{fd_total    = file_handle_cache:ulimit(),
+                   fhc_stats   = file_handle_cache_stats:get(),
+                   node_owners = sets:new()},
     %% If we emit an update straight away we will do so just before
     %% the mgmt db starts up - and then have to wait ?REFRESH_RATIO
     %% until we send another. So let's have a shorter wait in the hope
@@ -294,7 +343,39 @@ code_change(_, State, _) -> {ok, State}.
 
 %%--------------------------------------------------------------------
 
-emit_update(State) ->
+emit_update(State0) ->
+    State = update_state(State0),
     rabbit_event:notify(node_stats, infos(?KEYS, State)),
     erlang:send_after(?REFRESH_RATIO, self(), emit_update),
-    State.
+    emit_node_node_stats(State).
+
+emit_node_node_stats(State = #state{node_owners = Owners}) ->
+    Links = cluster_links(),
+    NewOwners = sets:from_list([{Node, Owner} || {Node, Owner, _} <- Links]),
+    Dead = sets:to_list(sets:subtract(Owners, NewOwners)),
+    [rabbit_event:notify(
+       node_node_deleted, [{route, Route}]) || {Node, _Owner} <- Dead,
+                                               Route <- [{node(), Node},
+                                                         {Node,   node()}]],
+    [rabbit_event:notify(
+       node_node_stats, [{route, {node(), Node}} | Stats]) ||
+        {Node, _Owner, Stats} <- Links],
+    State#state{node_owners = NewOwners}.
+
+update_state(State0 = #state{fhc_stats = FHC0}) ->
+    FHC = file_handle_cache_stats:get(),
+    Avgs = [{{Op, avg_time}, avg_op_time(Op, V, FHC, FHC0)}
+            || {{Op, time}, V} <- FHC],
+    State0#state{fhc_stats         = FHC,
+                 fhc_stats_derived = Avgs}.
+
+-define(MICRO_TO_MILLI, 1000).
+
+avg_op_time(Op, Time, FHC, FHC0) ->
+    Time0 = pget({Op, time}, FHC0),
+    TimeDelta = Time - Time0,
+    OpDelta = pget({Op, count}, FHC) - pget({Op, count}, FHC0),
+    case OpDelta of
+        0 -> 0;
+        _ -> (TimeDelta / OpDelta) / ?MICRO_TO_MILLI
+    end.
