@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is GoPivotal, Inc.
-%%   Copyright (c) 2010-2014 GoPivotal, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2015 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_wm_exchange).
@@ -55,9 +55,11 @@ accept_content(ReqData, Context) ->
       [{exchange, rabbit_mgmt_util:id(exchange, ReqData)}]).
 
 delete_resource(ReqData, Context) ->
+    IfUnused = "true" =:= wrq:get_qs_value("if-unused", ReqData),
     rabbit_mgmt_util:amqp_request(
       rabbit_mgmt_util:vhost(ReqData), ReqData, Context,
-      #'exchange.delete'{ exchange = id(ReqData) }).
+      #'exchange.delete'{exchange  = id(ReqData),
+                         if_unused = IfUnused}).
 
 is_authorized(ReqData, Context) ->
     rabbit_mgmt_util:is_authorized_vhost(ReqData, Context).
