@@ -20,68 +20,38 @@
 %%
 -module(rabbit_mgmt_test_clustering).
 
+-compile(export_all).
 -include("rabbit_mgmt_test.hrl").
 
--export([start_second_node/0, stop_second_node/0]).
-
 -import(rabbit_mgmt_test_http, [http_get/1, http_put/3, http_delete/2]).
 -import(rabbit_misc, [pget/2]).
 
 %%----------------------------------------------------------------------------
 
-start_second_node() -> start_other_node(hare, 5673),
-                       cluster_other_node(hare).
-stop_second_node()  -> stop_other_node(hare).
-
-start_other_node(Name, Port) ->
-    %% ?assertCmd seems to hang if you background anything. Bah!
-    Res = os:cmd("make -C " ++ plugin_dir() ++ " OTHER_NODE=" ++
-                     atom_to_list(Name) ++
-                     " OTHER_PORT=" ++ integer_to_list(Port) ++
-                     " start-other-node ; echo $?"),
-    LastLine = hd(lists:reverse(string:tokens(Res, "\n"))),
-    case LastLine of
-        "0" -> ok;
-        _   -> ?debugVal(Res),
-               ?assertEqual("0", LastLine)
-    end.
-
-cluster_other_node(Name) ->
-    ?assertCmd("make -C " ++ plugin_dir() ++ " OTHER_NODE=" ++
-                   atom_to_list(Name) ++ " cluster-other-node").
-
-stop_other_node(Name) ->
-    ?assertCmd("make -C " ++ plugin_dir() ++ " OTHER_NODE=" ++
-                   atom_to_list(Name) ++ " stop-other-node").
-
-plugin_dir() ->
-    {ok, [[File]]} = init:get_argument(config),
-    filename:dirname(filename:dirname(File)).
-
-%%----------------------------------------------------------------------------
-
-cluster_nodes_test() ->
+cluster_nodes_with() -> cluster_ab.
+cluster_nodes([_A, _B]) ->
     ?assertEqual(2, length(http_get("/nodes"))),
     ok.
 
-ha_test_() ->
-    {timeout, 120, fun ha/0}.
-
-ha() ->
+ha_with() -> cluster_ab.
+ha([RabbitCfg, HareCfg]) ->
+    Rabbit = pget(nodename, RabbitCfg),
+    Hare = pget(nodename, HareCfg),
     Policy = [{pattern,    <<".*">>},
               {definition, [{'ha-mode', <<"all">>}]}],
     http_put("/policies/%2f/HA", Policy, ?NO_CONTENT),
-    QArgs = [{node, <<"hare">>}],
+    QArgs = [{node, list_to_binary(atom_to_list(Hare))}],
     http_put("/queues/%2f/ha-queue", QArgs, ?NO_CONTENT),
     Q = wait_for("/queues/%2f/ha-queue"),
-    assert_node(hare, pget(node, Q)),
-    assert_single_node('rabbit-test', pget(slave_nodes, Q)),
-    assert_single_node('rabbit-test', pget(synchronised_slave_nodes, Q)),
-    restart_node(),
+    assert_node(Hare, pget(node, Q)),
+    assert_single_node(Rabbit, pget(slave_nodes, Q)),
+    assert_single_node(Rabbit, pget(synchronised_slave_nodes, Q)),
+    _HareCfg2 = rabbit_test_configs:restart_node(HareCfg),
+
     Q2 = wait_for("/queues/%2f/ha-queue"),
-    assert_node('rabbit-test', pget(node, Q2)),
-    assert_single_node(hare, pget(slave_nodes, Q2)),
-    assert_single_node(hare, pget(synchronised_slave_nodes, Q2)),
+    assert_node(Rabbit, pget(node, Q2)),
+    assert_single_node(Hare, pget(slave_nodes, Q2)),
+    assert_single_node(Hare, pget(synchronised_slave_nodes, Q2)),
     http_delete("/queues/%2f/ha-queue", ?NO_CONTENT),
     http_delete("/policies/%2f/HA", ?NO_CONTENT),
     ok.
@@ -126,8 +96,3 @@ assert_node(Exp, Act) ->
 
 read_node(N) ->
     list_to_atom(hd(string:tokens(binary_to_list(N), "@"))).
-
-restart_node() ->
-    stop_second_node(),
-    timer:sleep(1000),
-    start_second_node().
