@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2015 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mqtt_reader).
@@ -52,7 +52,7 @@ handle_cast({go, Sock0, SockTransform, KeepaliveSup}, undefined) ->
                 {ok, Sock} ->
                     rabbit_alarm:register(
                       self(), {?MODULE, conserve_resources, []}),
-                    ProcessorState = rabbit_mqtt_processor:initial_state(Sock),
+                    ProcessorState = rabbit_mqtt_processor:initial_state(Sock,ssl_login_name(Sock)),
                     {noreply,
                      control_throttle(
                        #state{socket           = Sock,
@@ -138,9 +138,10 @@ handle_info({start_keepalives, Keepalive},
                     KeepaliveSup, Sock, 0, SendFun, Keepalive, ReceiveFun),
     {noreply, State #state { keepalive = Heartbeater }};
 
-handle_info(keepalive_timeout, State = #state { conn_name = ConnStr }) ->
+handle_info(keepalive_timeout, State = #state {conn_name = ConnStr,
+                                               proc_state = PState}) ->
     log(error, "closing MQTT connection ~p (keepalive timeout)~n", [ConnStr]),
-    {stop, {shutdown, keepalive_timeout}, State};
+    send_will_and_terminate(PState, {shutdown, keepalive_timeout}, State);
 
 handle_info(Msg, State) ->
     {stop, {mqtt_unexpected_msg, Msg}, State}.
@@ -191,6 +192,17 @@ terminate(_Reason, #state{proc_state = ProcState}) ->
 code_change(_OldVsn, State, _Extra) ->
     {ok, State}.
 
+ssl_login_name(Sock) ->
+  case rabbit_net:peercert(Sock) of
+      {ok, C}              -> case rabbit_ssl:peer_cert_auth_name(C) of
+                                    unsafe    -> none;
+                                    not_found -> none;
+                                    Name      -> Name
+                                end;
+      {error, no_peercert} -> none;
+      nossl                -> none
+  end.
+
 %%----------------------------------------------------------------------------
 
 process_received_bytes(<<>>, State) ->
@@ -245,9 +257,12 @@ pstate(State = #state {}, PState = #proc_state{}) ->
 log(Level, Fmt, Args) -> rabbit_log:log(connection, Level, Fmt, Args).
 
 send_will_and_terminate(PState, State) ->
+    send_will_and_terminate(PState, {shutdown, conn_closed}, State).
+
+send_will_and_terminate(PState, Reason, State) ->
     rabbit_mqtt_processor:send_will(PState),
     % todo: flush channel after publish
-    {stop, {shutdown, conn_closed}, State}.
+    {stop, Reason, State}.
 
 network_error(closed,
               State = #state{ conn_name  = ConnStr,
