@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2015 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mqtt_sup).
@@ -32,7 +32,7 @@ init([{Listeners, SslListeners0}]) ->
         = case SslListeners0 of
               [] -> {none, []};
               _  -> {rabbit_networking:ensure_ssl(),
-                     case rabbit_networking:poodle_check('STOMP') of
+                     case rabbit_networking:poodle_check('MQTT') of
                          ok     -> SslListeners0;
                          danger -> []
                      end}
