@@ -45,6 +45,16 @@ class TestLifecycle(base.BaseTest):
         d = "/queue/unsub04"
         self.unsub_test(d, self.sub_and_send(d, subid="queid", receipt="unsub.rct"), numRcts=1)
 
+    def test_connect_version_1_0(self):
+        ''' Test CONNECT with version 1.0'''
+        self.conn.disconnect()
+        new_conn = self.create_connection(version="1.0")
+        try:
+            self.assertTrue(new_conn.is_connected())
+        finally:
+            new_conn.disconnect()
+            self.assertFalse(new_conn.is_connected())
+
     def test_connect_version_1_1(self):
         ''' Test CONNECT with version 1.1'''
         self.conn.disconnect()
@@ -55,10 +65,20 @@ class TestLifecycle(base.BaseTest):
             new_conn.disconnect()
             self.assertFalse(new_conn.is_connected())
 
+    def test_connect_version_1_2(self):
+        ''' Test CONNECT with version 1.2'''
+        self.conn.disconnect()
+        new_conn = self.create_connection(version="1.2")
+        try:
+            self.assertTrue(new_conn.is_connected())
+        finally:
+            new_conn.disconnect()
+            self.assertFalse(new_conn.is_connected())
+
     def test_heartbeat_disconnects_client(self):
         ''' Test heart-beat disconnection'''
         self.conn.disconnect()
-        new_conn = self.create_connection(heartbeat="1500,0")
+        new_conn = self.create_connection(version='1.1', heartbeats=(1500, 0))
         try:
             self.assertTrue(new_conn.is_connected())
             time.sleep(1)
@@ -71,37 +91,28 @@ class TestLifecycle(base.BaseTest):
 
     def test_unsupported_version(self):
         ''' Test unsupported version on CONNECT command'''
-        self.bad_connect(stomp.Connection(user="guest",
-                                          passcode="guest",
-                                          version="100.1"),
-                         "Supported versions are 1.0,1.1,1.2\n")
+        self.bad_connect("Supported versions are 1.0,1.1,1.2\n", version='100.1')
 
     def test_bad_username(self):
         ''' Test bad username'''
-        self.bad_connect(stomp.Connection(user="gust",
-                                          passcode="guest"),
-                         "Access refused for user 'gust'\n")
+        self.bad_connect("Access refused for user 'gust'\n", user='gust')
 
     def test_bad_password(self):
         ''' Test bad password'''
-        self.bad_connect(stomp.Connection(user="guest",
-                                          passcode="gust"),
-                         "Access refused for user 'guest'\n")
+        self.bad_connect("Access refused for user 'guest'\n", passcode='gust')
 
     def test_bad_vhost(self):
         ''' Test bad virtual host'''
-        self.bad_connect(stomp.Connection(user="guest",
-                                          passcode="guest",
-                                          virtual_host="//"),
-                         "Virtual host '//' access denied")
+        self.bad_connect("Virtual host '//' access denied", version='1.1', vhost='//')
 
-    def bad_connect(self, new_conn, expected):
+    def bad_connect(self, expected, user='guest', passcode='guest', **kwargs):
         self.conn.disconnect()
+        new_conn = self.create_connection_obj(**kwargs)
         listener = base.WaitableListener()
         new_conn.set_listener('', listener)
         try:
             new_conn.start()
-            new_conn.connect()
+            new_conn.connect(user, passcode)
             self.assertTrue(listener.await())
             self.assertEquals(expected, listener.errors[0]['message'])
         finally:
@@ -136,7 +147,7 @@ class TestLifecycle(base.BaseTest):
 
     def unsub_test(self, dest, verbs, numRcts=0):
         def afterfun():
-            self.conn.send("after-test", destination=dest)
+            self.conn.send(dest, "after-test")
         subverb, unsubverb = verbs
         self.assertListenerAfter(subverb, numMsgs=1,
                            errMsg="FAILED to subscribe and send")
@@ -145,20 +156,13 @@ class TestLifecycle(base.BaseTest):
         self.assertListenerAfter(afterfun,
                            errMsg="Still receiving messages")
 
-    def sub_and_send(self, dest, subid="", receipt=""):
+    def sub_and_send(self, dest, subid=None, receipt=None):
         def subfun():
-            if subid=="":
-                self.conn.subscribe(destination=dest)
-            else:
-                self.conn.subscribe(destination=dest, id=subid)
-            self.conn.send("test", destination=dest)
+            self.subscribe_dest(self.conn, dest, subid)
+            self.conn.send(dest, "test")
         def unsubfun():
-            if subid=="" and receipt=="":
-                self.conn.unsubscribe(destination=dest)
-            elif receipt=="":
-                self.conn.unsubscribe(id=subid)
-            elif subid=="":
-                self.conn.unsubscribe(destination=dest, receipt=receipt)
-            else:
-                self.conn.unsubscribe(id=subid, receipt=receipt)
+            headers = {}
+            if receipt != None:
+                headers['receipt'] = receipt
+            self.unsubscribe_dest(self.conn, dest, subid, **headers)
         return subfun, unsubfun
