@@ -1,24 +1,29 @@
 import unittest
 import os
+import os.path
+import sys
 
 import stomp
 import base
+import ssl
 
-ssl_key_file = os.path.abspath("test/certs/client/key.pem")
-ssl_cert_file = os.path.abspath("test/certs/client/cert.pem")
-ssl_ca_certs = os.path.abspath("test/certs/testca/cacert.pem")
+
+base_path = os.path.dirname(sys.argv[0])
+
+ssl_key_file = os.path.abspath(base_path + "/../certs/client/key.pem")
+ssl_cert_file = os.path.abspath(base_path + "/../certs/client/cert.pem")
+ssl_ca_certs = os.path.abspath(base_path  + "/../certs/testca/cacert.pem")
 
 class TestSslClient(unittest.TestCase):
 
     def __ssl_connect(self):
-        conn = stomp.Connection(user="guest", passcode="guest",
-                                host_and_ports = [ ('localhost', 61614) ],
+        conn = stomp.Connection(host_and_ports = [ ('localhost', 61614) ],
                                 use_ssl = True, ssl_key_file = ssl_key_file,
                                 ssl_cert_file = ssl_cert_file,
                                 ssl_ca_certs = ssl_ca_certs)
-
+        print "FILE: ", ssl_cert_file
         conn.start()
-        conn.connect()
+        conn.connect("guest", "guest")
         return conn
 
     def __ssl_auth_connect(self):
@@ -32,11 +37,11 @@ class TestSslClient(unittest.TestCase):
 
     def test_ssl_connect(self):
         conn = self.__ssl_connect()
-        conn.stop()
+        conn.disconnect()
 
     def test_ssl_auth_connect(self):
         conn = self.__ssl_auth_connect()
-        conn.stop()
+        conn.disconnect()
 
     def test_ssl_send_receive(self):
         conn = self.__ssl_connect()
@@ -53,7 +58,7 @@ class TestSslClient(unittest.TestCase):
             conn.set_listener('', listener)
 
             d = "/topic/ssl.test"
-            conn.subscribe(destination=d, receipt="sub")
+            conn.subscribe(destination=d, ack="auto", id="ctag", receipt="sub")
 
             self.assertTrue(listener.await(1))
 
@@ -61,7 +66,7 @@ class TestSslClient(unittest.TestCase):
                               listener.receipts[0]['headers']['receipt-id'])
 
             listener.reset(1)
-            conn.send("Hello SSL!", destination=d)
+            conn.send(body="Hello SSL!", destination=d)
 
             self.assertTrue(listener.await())
 
